"""
File containing all functions generally used for any file type

Authors:
    Marie FU
"""

from io import BufferedReader
import re
import chardet
from streamlit.runtime.uploaded_file_manager import UploadedFile
import streamlit as st


# Define constant for file encoding supported (name in lower case)
FILE_ENCODING = ["utf-8", "utf-8-sig", "ascii"]


def toStartOfFile(file : UploadedFile) -> None :

    """
    Goes back to the start of the file

    Args:
        file (UploadedFile) : target file

    Returns:
        None

    Authors:
        Marie FU
    """

    file.seek(0)
    
    return


def checkEncoding(file : UploadedFile|str) -> str|None:

    """
    Checks if a given file is encoded correctly

    Args:
        file (UploadedFile) : file to check

    Returns:
        (String or None) : file encoding or None

    Authors:
        Nicolas Atrax
        Marie FU
    """

    content = b""
    if isinstance(file, UploadedFile):
        content = file.read()
    elif isinstance(file, str):
        with open(file, "rb") as f:
            content = f.read()
    encoding = chardet.detect(content)["encoding"]
    if encoding is not None:
        if encoding.lower() in FILE_ENCODING:
            return encoding
    else:
        for decoding in FILE_ENCODING:
            try:
                content.decode(decoding)
                encoding = decoding
                break
            except UnicodeDecodeError:
                print("failed: "+decoding)
    return None


def checkNewFileName(newFileName : str, fileName : str) -> str|None :

    """
    Check if file name is correct.

    Args:
        newFileName (String) : new file name to check
        fileName (String) : current file name

    Returns:
        (String or None) : File name or None

    Authors:
        Marie FU
    """

    if newFileName :
        if re.match((r'^[a-zA-Z0-9_À-ÖÙ-öù-ÿ]+$'),newFileName) :
            return f"{newFileName}.tsv"
        else :
            return None
    else :
        return f"{fileName.split('.')[0]}.tsv"


def lowerName(name : str) -> str:

    """
    Set a given String to lowercase and removes unecessary blank spaces

    Args:
        name (String) : The given string to format

    Returns:
        res (String) : The changed string

    Authors:
        Marie FU
    """
    
    res = " ".join(name.split()).lower()
    if "_" in res:
        res = " ".join(res.split("_"))
    return res


@st.experimental_fragment
def errDisplay(errMessageLog : str) -> None:

    """
    Display the error messages log in a container

    Args:
        errMessageLog (String) : error message

    Returns:
        None

    Authors:
        Marie FU
    """

    with st.expander(st.session_state.general_text_dict['see_err']):
        st.download_button("errorLog", errMessageLog, "errorLog")
        st.write(errMessageLog)

    return


@st.experimental_fragment
def show_download_button(newFileName : str, fileData : str|bytes) -> None:

    """
    Display button to download file (this will not reload the page)

    Args:
        newFileName (String) : Button label and file name
        filedata (String) : File data

    Returns:
        None

    Authors:
        Marie FU
    """

    with st.container() :
        st.write(st.session_state.general_text_dict['new_file'])
        if isinstance(fileData, str):
            st.download_button(newFileName, fileData.replace("\"","”"), newFileName)
        else:
            st.download_button(newFileName, fileData, newFileName)
        
    return
